"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var ChainGunLink_1 = require("./ChainGunLink");
var GunGraphUtils_1 = require("./Graph/GunGraphUtils");
var GunGraph_1 = require("./Graph/GunGraph");
var WebSocketGraphConnector_1 = require("./Transports/WebSocketGraphConnector");
/**
 * Main entry point for ChainGun
 *
 * Usage:
 *
 *   const gun = new ChainGun({ peers: ["https://notabug.io/gun"]})
 *   gun.get("nab/things/59382d2a08b7d7073415b5b6ae29dfe617690d74").on(thing => console.log(this))
 */
var ChainGun = /** @class */ (function () {
    function ChainGun(opt, LinkClass) {
        if (LinkClass === void 0) { LinkClass = ChainGunLink_1.ChainGunLink; }
        if (opt && opt.graph) {
            this.graph = opt.graph;
        }
        else {
            this.graph = new GunGraph_1.GunGraph();
            this.graph.use(GunGraphUtils_1.diffGunCRDT);
            this.graph.use(GunGraphUtils_1.diffGunCRDT, 'write');
        }
        this._opt = {};
        if (opt)
            this.opt(opt);
        this.LinkClass = LinkClass;
    }
    /**
     * Set ChainGun configuration options
     *
     * @param options
     */
    ChainGun.prototype.opt = function (options) {
        var _this = this;
        this._opt = __assign(__assign({}, this._opt), options);
        if (options.peers) {
            options.peers.forEach(function (peer) {
                return _this.graph.connect(new WebSocketGraphConnector_1.WebSocketGraphConnector(peer, _this._opt.WS));
            });
        }
        return this;
    };
    /**
     * Traverse a location in the graph
     *
     * @param key Key to read data from
     * @param cb
     * @returns New chain context corresponding to given key
     */
    ChainGun.prototype.get = function (soul, cb) {
        return new this.LinkClass(this, soul);
    };
    return ChainGun;
}());
exports.ChainGun = ChainGun;
//# sourceMappingURL=ChainGun.js.map