"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var GunEvent_1 = require("../ControlFlow/GunEvent");
var GunGraphUtils_1 = require("./GunGraphUtils");
var GunGraphNode_1 = require("./GunGraphNode");
/**
 * High level management of a subset of the gun graph
 *
 * Provides facilities for querying and writing to graph data from one or more sources
 */
var GunGraph = /** @class */ (function () {
    function GunGraph() {
        this.id = GunGraphUtils_1.generateMessageId();
        this._receiveGraphData = this._receiveGraphData.bind(this);
        this.__onConnectorStatus = this.__onConnectorStatus.bind(this);
        this.activeConnectors = 0;
        this.graphData = new GunEvent_1.GunEvent('graph data');
        this._opt = {};
        this._graph = {};
        this._nodes = {};
        this._connectors = [];
        this._readMiddleware = [];
        this._writeMiddleware = [];
    }
    /**
     * Configure graph options
     *
     * Currently unused
     *
     * @param options
     */
    GunGraph.prototype.opt = function (options) {
        this._opt = __assign(__assign({}, this._opt), options);
        return this;
    };
    /**
     * Connect to a source/destination for graph data
     *
     * @param connector the source or destination for graph data
     */
    GunGraph.prototype.connect = function (connector) {
        if (this._connectors.indexOf(connector) !== -1)
            return this;
        if (connector.isConnected)
            this.activeConnectors++;
        connector.events.connection.on(this.__onConnectorStatus);
        connector.events.graphData.on(this._receiveGraphData);
        this._connectors.push(connector);
        return this;
    };
    /**
     * Disconnect from a source/destination for graph data
     *
     * @param connector the source or destination for graph data
     */
    GunGraph.prototype.disconnect = function (connector) {
        var idx = this._connectors.indexOf(connector);
        connector.events.graphData.off(this._receiveGraphData);
        connector.events.connection.off(this.__onConnectorStatus);
        if (idx !== -1)
            this._connectors.splice(idx, 1);
        if (connector.isConnected)
            this.activeConnectors--;
        return this;
    };
    /**
     * Register graph middleware
     *
     * @param middleware The middleware function to add
     * @param kind Optionaly register write middleware instead of read by passing "write"
     */
    GunGraph.prototype.use = function (middleware, kind) {
        if (kind === void 0) { kind = 'read'; }
        if (kind === 'read') {
            this._readMiddleware.push(middleware);
        }
        else if (kind === 'write') {
            this._writeMiddleware.push(middleware);
        }
        return this;
    };
    /**
     * Unregister graph middleware
     *
     * @param middleware The middleware function to remove
     * @param kind Optionaly unregister write middleware instead of read by passing "write"
     */
    GunGraph.prototype.unuse = function (middleware, kind) {
        if (kind === void 0) { kind = 'read'; }
        if (kind === 'read') {
            var idx = this._readMiddleware.indexOf(middleware);
            if (idx !== -1)
                this._readMiddleware.splice(idx, 1);
        }
        else if (kind === 'write') {
            var idx = this._writeMiddleware.indexOf(middleware);
            if (idx !== -1)
                this._writeMiddleware.splice(idx, 1);
        }
        return this;
    };
    /**
     * Read a potentially multi-level deep path from the graph
     *
     * @param path The path to read
     * @param cb The callback to invoke with results
     * @returns a cleanup function to after done with query
     */
    GunGraph.prototype.query = function (path, cb) {
        var _this = this;
        var lastSouls = [];
        var currentValue;
        var updateQuery = function () {
            var _a = GunGraphUtils_1.getPathData(path, _this._graph), souls = _a.souls, value = _a.value, complete = _a.complete;
            var _b = GunGraphUtils_1.diffSets(lastSouls, souls), added = _b[0], removed = _b[1];
            if ((complete && typeof currentValue === 'undefined') ||
                (typeof value !== 'undefined' && value !== currentValue)) {
                currentValue = value;
                cb(value, path[path.length - 1]);
            }
            for (var i = 0; i < added.length; i++) {
                _this._requestSoul(added[i], updateQuery);
            }
            for (var i = 0; i < removed.length; i++) {
                _this._unlistenSoul(removed[i], updateQuery);
            }
            lastSouls = souls;
        };
        updateQuery();
        return function () {
            for (var i = 0; i < lastSouls.length; i++) {
                _this._unlistenSoul(lastSouls[i], updateQuery);
            }
        };
    };
    /**
     * Write graph data to a potentially multi-level deep path in the graph
     *
     * @param path The path to read
     * @param data The value to write
     * @param cb Callback function to be invoked for write acks
     * @returns a cleanup function to after done with query
     */
    GunGraph.prototype.putPath = function (path, data, cb) {
        var _a;
        var _this = this;
        if (path.length === 1) {
            if (data && typeof data === 'object') {
                this.put((_a = {},
                    _a[path[0]] = data,
                    _a), cb);
            }
            return;
        }
        var lastKey = path[path.length - 1];
        var parentPath = path.slice(0, path.length - 1);
        var lastSouls = [];
        var end = function () {
            for (var i = 0; i < lastSouls.length; i++) {
                _this._unlistenSoul(lastSouls[i], updateQuery);
            }
            lastSouls = [];
        };
        var updateQuery = function () {
            var _a;
            var _b = GunGraphUtils_1.getPathData(path, _this._graph), souls = _b.souls, complete = _b.complete;
            var _c = GunGraphUtils_1.diffSets(lastSouls, souls), added = _c[0], removed = _c[1];
            if (complete) {
                if (souls.length === parentPath.length) {
                    var lastSoul = souls[souls.length - 1];
                    _this.putPath([lastSoul], (_a = {}, _a[lastKey] = data, _a), cb);
                }
                else {
                    throw new Error('Deep puts only partially supported');
                }
                end();
            }
            else {
                for (var i = 0; i < added.length; i++) {
                    _this._requestSoul(added[i], updateQuery);
                }
                for (var i = 0; i < removed.length; i++) {
                    _this._unlistenSoul(removed[i], updateQuery);
                }
            }
            lastSouls = souls;
        };
        updateQuery();
        return end;
    };
    /**
     * Write node data
     *
     * @param data one or more gun nodes keyed by soul
     */
    GunGraph.prototype.put = function (data, cb) {
        return __awaiter(this, void 0, void 0, function () {
            var diff, i, msgId, i, souls, i;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        diff = GunGraphUtils_1.flattenGraphData(GunGraphUtils_1.addMissingState(data));
                        i = 0;
                        _a.label = 1;
                    case 1:
                        if (!(i < this._writeMiddleware.length)) return [3 /*break*/, 4];
                        if (!diff)
                            return [2 /*return*/];
                        return [4 /*yield*/, this._writeMiddleware[i](diff, this._graph)];
                    case 2:
                        diff = _a.sent();
                        _a.label = 3;
                    case 3:
                        i++;
                        return [3 /*break*/, 1];
                    case 4:
                        if (!diff)
                            return [2 /*return*/];
                        msgId = GunGraphUtils_1.generateMessageId();
                        for (i = 0; i < this._connectors.length; i++) {
                            this._connectors[i].put({
                                msgId: msgId,
                                graph: diff,
                                cb: cb
                            });
                        }
                        souls = Object.keys(diff);
                        for (i = 0; i < souls.length; i++) {
                            this._node(souls[i]);
                        }
                        this._receiveGraphData(diff);
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Synchronously invoke callback function for each connector to this graph
     *
     * @param cb The callback to invoke
     */
    GunGraph.prototype.eachConnector = function (cb) {
        for (var i = 0; i < this._connectors.length; i++) {
            cb(this._connectors[i]);
        }
        return this;
    };
    /**
     * Update graph data in this chain from some local or external source
     *
     * @param data node data to include
     */
    GunGraph.prototype._receiveGraphData = function (data, id, replyToId) {
        return __awaiter(this, void 0, void 0, function () {
            var diff, i, souls, i, soul, node, nodeData;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        diff = data;
                        i = 0;
                        _a.label = 1;
                    case 1:
                        if (!(i < this._readMiddleware.length)) return [3 /*break*/, 4];
                        if (!diff)
                            return [2 /*return*/];
                        return [4 /*yield*/, this._readMiddleware[i](diff, this._graph)];
                    case 2:
                        diff = _a.sent();
                        _a.label = 3;
                    case 3:
                        i++;
                        return [3 /*break*/, 1];
                    case 4:
                        if (!diff)
                            return [2 /*return*/];
                        souls = Object.keys(diff);
                        for (i = 0; i < souls.length; i++) {
                            soul = souls[i];
                            node = this._nodes[soul];
                            if (!node) {
                                this._forgetSoul(soul);
                                continue;
                            }
                            nodeData = (this._graph[soul] = GunGraphUtils_1.mergeGunNodes(this._graph[soul], diff[soul]));
                            if (node)
                                node.receive(nodeData);
                        }
                        this.graphData.trigger(diff, id, replyToId);
                        return [2 /*return*/];
                }
            });
        });
    };
    GunGraph.prototype._node = function (soul) {
        return (this._nodes[soul] =
            this._nodes[soul] || new GunGraphNode_1.GunGraphNode(this, soul, this._receiveGraphData));
    };
    GunGraph.prototype._requestSoul = function (soul, cb) {
        this._node(soul).get(cb);
        return this;
    };
    GunGraph.prototype._unlistenSoul = function (soul, cb) {
        var node = this._nodes[soul];
        if (!node)
            return this;
        node.off(cb);
        if (node.listenerCount() <= 0) {
            node.off();
            this._forgetSoul(soul);
        }
        return this;
    };
    GunGraph.prototype._forgetSoul = function (soul) {
        var node = this._nodes[soul];
        if (node) {
            node.off();
            delete this._nodes[soul];
        }
        delete this._graph[soul];
        return this;
    };
    GunGraph.prototype.__onConnectorStatus = function (connected) {
        if (connected) {
            this.activeConnectors++;
        }
        else {
            this.activeConnectors--;
        }
    };
    return GunGraph;
}());
exports.GunGraph = GunGraph;
//# sourceMappingURL=GunGraph.js.map