"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var GunGraphConnector_1 = require("./GunGraphConnector");
var GunGraphUtils_1 = require("../Graph/GunGraphUtils");
var GunGraphWireConnector = /** @class */ (function (_super) {
    __extends(GunGraphWireConnector, _super);
    function GunGraphWireConnector(name) {
        if (name === void 0) { name = 'GunWireProtocol'; }
        var _this = _super.call(this, name) || this;
        _this._callbacks = {};
        _this._onProcessedInput = _this._onProcessedInput.bind(_this);
        _this.inputQueue.completed.on(_this._onProcessedInput);
        return _this;
    }
    /**
     * Send graph data for one or more nodes
     *
     * @returns A function to be called to clean up callback listeners
     */
    GunGraphWireConnector.prototype.put = function (_a) {
        var graph = _a.graph, _b = _a.msgId, msgId = _b === void 0 ? '' : _b, _c = _a.replyTo, replyTo = _c === void 0 ? '' : _c, cb = _a.cb;
        if (!graph)
            return function () { };
        var msg = {
            put: graph
        };
        if (msgId)
            msg['#'] = msgId;
        if (replyTo)
            msg['@'] = replyTo;
        return this.req(msg, cb);
    };
    /**
     * Request data for a given soul
     *
     * @returns A function to be called to clean up callback listeners
     */
    GunGraphWireConnector.prototype.get = function (_a) {
        var soul = _a.soul, cb = _a.cb, _b = _a.msgId, msgId = _b === void 0 ? '' : _b, _c = _a.key // TODO
        , key = _c === void 0 ? '' : _c // TODO
        ;
        var get = { '#': soul };
        // if (key) get["."] = key
        var msg = { get: get };
        if (msgId)
            msg['#'] = msgId;
        return this.req(msg, cb);
    };
    /**
     * Send a message that expects responses via @
     *
     * @param msg
     * @param cb
     */
    GunGraphWireConnector.prototype.req = function (msg, cb) {
        var _this = this;
        var now = new Date().getTime();
        var done;
        var reqId = (msg['#'] = msg['#'] || GunGraphUtils_1.generateMessageId());
        var cbWrap = function (resp) {
            if (!done) {
                done = new Date().getTime();
                var duration = done - now;
                if (duration > 500) {
                    console.log('slow req', duration, msg.put ? Object.keys(msg.put) : msg);
                }
            }
            if (cb)
                return cb(resp);
        };
        if (cb)
            this._callbacks[reqId] = cbWrap;
        this.send([msg]);
        return function () {
            delete _this._callbacks[reqId];
        };
    };
    GunGraphWireConnector.prototype._onProcessedInput = function (msg) {
        if (!msg)
            return;
        var id = msg['#'];
        var replyTo = msg['@'];
        if ('put' in msg) {
            if (msg.put)
                this.events.graphData.trigger(msg.put, id, replyTo);
        }
        if (replyTo) {
            var cb = this._callbacks[replyTo];
            if (cb)
                cb(msg);
        }
        this.events.receiveMessage.trigger(msg);
    };
    return GunGraphWireConnector;
}(GunGraphConnector_1.GunGraphConnector));
exports.GunGraphWireConnector = GunGraphWireConnector;
//# sourceMappingURL=GunGraphWireConnector.js.map