import { GunEvent } from '../ControlFlow/GunEvent';
import { GunGraphConnector } from '../Transports/GunGraphConnector';
interface GunGraphOptions {
}
/**
 * High level management of a subset of the gun graph
 *
 * Provides facilities for querying and writing to graph data from one or more sources
 */
export declare class GunGraph {
    id: string;
    graphData: GunEvent<GunGraphData, string | undefined, string | undefined>;
    activeConnectors: number;
    private _opt;
    private _connectors;
    private _readMiddleware;
    private _writeMiddleware;
    private _graph;
    private _nodes;
    constructor();
    /**
     * Configure graph options
     *
     * Currently unused
     *
     * @param options
     */
    opt(options: GunGraphOptions): this;
    /**
     * Connect to a source/destination for graph data
     *
     * @param connector the source or destination for graph data
     */
    connect(connector: GunGraphConnector): this;
    /**
     * Disconnect from a source/destination for graph data
     *
     * @param connector the source or destination for graph data
     */
    disconnect(connector: GunGraphConnector): this;
    /**
     * Register graph middleware
     *
     * @param middleware The middleware function to add
     * @param kind Optionaly register write middleware instead of read by passing "write"
     */
    use(middleware: ChainGunMiddleware, kind?: ChainGunMiddlewareType): this;
    /**
     * Unregister graph middleware
     *
     * @param middleware The middleware function to remove
     * @param kind Optionaly unregister write middleware instead of read by passing "write"
     */
    unuse(middleware: ChainGunMiddleware, kind?: ChainGunMiddlewareType): this;
    /**
     * Read a potentially multi-level deep path from the graph
     *
     * @param path The path to read
     * @param cb The callback to invoke with results
     * @returns a cleanup function to after done with query
     */
    query(path: string[], cb: GunOnCb): () => void;
    /**
     * Write graph data to a potentially multi-level deep path in the graph
     *
     * @param path The path to read
     * @param data The value to write
     * @param cb Callback function to be invoked for write acks
     * @returns a cleanup function to after done with query
     */
    putPath(path: string[], data: GunValue, cb?: GunMsgCb): (() => void) | undefined;
    /**
     * Write node data
     *
     * @param data one or more gun nodes keyed by soul
     */
    put(data: GunGraphData, cb?: GunMsgCb): Promise<void>;
    /**
     * Synchronously invoke callback function for each connector to this graph
     *
     * @param cb The callback to invoke
     */
    eachConnector(cb: (connector: GunGraphConnector) => void): this;
    /**
     * Update graph data in this chain from some local or external source
     *
     * @param data node data to include
     */
    private _receiveGraphData;
    private _node;
    private _requestSoul;
    private _unlistenSoul;
    private _forgetSoul;
    private __onConnectorStatus;
}
export {};
